# -*- coding: utf-8 -*-


class HealthCardException(Exception):
    pass


class HealthCardSelectException(HealthCardException):
    pass


class HealthCardReadException(HealthCardException):
    pass
